import { IQueryHandler, QueryHandler } from '@nestjs/cqrs';
import { MatchQuery } from './match.query';
import { Algorithm } from './algorithm.abstract';
import { PassengerOrientedAlgorithm } from './passenger-oriented-algorithm';
import { AlgorithmType } from '../../types/algorithm.types';
import { Inject } from '@nestjs/common';
import { AdRepositoryPort } from '@modules/ad/core/application/ports/ad.repository.port';
import { AD_REPOSITORY, PARAMS_PROVIDER } from '@modules/ad/ad.di-tokens';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';
import { DefaultParamsProviderPort } from '../../ports/default-params-provider.port';
import { DefaultParams } from '../../ports/default-params.type';

@QueryHandler(MatchQuery)
export class MatchQueryHandler implements IQueryHandler {
  private readonly _defaultParams: DefaultParams;

  constructor(
    @Inject(PARAMS_PROVIDER)
    private readonly defaultParamsProvider: DefaultParamsProviderPort,
    @Inject(AD_REPOSITORY) private readonly repository: AdRepositoryPort,
  ) {
    this._defaultParams = defaultParamsProvider.getParams();
  }

  execute = async (query: MatchQuery): Promise<MatchEntity[]> => {
    query
      .setMissingMarginDurations(this._defaultParams.DEPARTURE_TIME_MARGIN)
      .setMissingStrict(this._defaultParams.STRICT)
      .setDefaultDriverAndPassengerParameters({
        driver: this._defaultParams.DRIVER,
        passenger: this._defaultParams.PASSENGER,
        seatsProposed: this._defaultParams.SEATS_PROPOSED,
        seatsRequested: this._defaultParams.SEATS_REQUESTED,
      })
      .setDefaultAlgorithmParameters({
        algorithmType: this._defaultParams.ALGORITHM_TYPE,
        remoteness: 0,
        useProportion: false,
        proportion: 0,
        useAzimuth: false,
        azimuthMargin: 0,
        maxDetourDistanceRatio: 0,
        maxDetourDurationRatio: 0,
      });

    let algorithm: Algorithm;
    switch (query.algorithmType) {
      case AlgorithmType.PASSENGER_ORIENTED:
      default:
        algorithm = new PassengerOrientedAlgorithm(query, this.repository);
    }
    return algorithm.match();
  };
}
