import { DefaultParams } from '@modules/ad/core/application/ports/default-params.type';
import { DefaultParamsProvider } from '@modules/ad/infrastructure/default-params-provider';
import { ConfigService } from '@nestjs/config';
import { Test, TestingModule } from '@nestjs/testing';

const mockConfigService = {
  get: jest.fn().mockImplementation((value: string) => {
    switch (value) {
      case 'DEPARTURE_TIME_MARGIN':
        return 900;
      case 'ROLE':
        return 'passenger';
      case 'SEATS_PROPOSED':
        return 3;
      case 'SEATS_REQUESTED':
        return 1;
      case 'STRICT_FREQUENCY':
        return 'false';
      case 'TIMEZONE':
        return 'Europe/Paris';
      case 'ALGORITHM_TYPE':
        return 'PASSENGER_ORIENTED';
      case 'REMOTENESS':
        return 15000;
      case 'USE_PROPORTION':
        return 'true';
      case 'PROPORTION':
        return 0.3;
      case 'USE_AZIMUTH':
        return 'true';
      case 'AZIMUTH_MARGIN':
        return 10;
      case 'MAX_DETOUR_DISTANCE_RATIO':
        return 0.3;
      case 'MAX_DETOUR_DURATION_RATIO':
        return 0.3;
      default:
        return 'some_default_value';
    }
  }),
};

describe('DefaultParamsProvider', () => {
  let defaultParamsProvider: DefaultParamsProvider;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        DefaultParamsProvider,
        {
          provide: ConfigService,
          useValue: mockConfigService,
        },
      ],
    }).compile();

    defaultParamsProvider = module.get<DefaultParamsProvider>(
      DefaultParamsProvider,
    );
  });

  it('should be defined', () => {
    expect(defaultParamsProvider).toBeDefined();
  });

  it('should provide default params', async () => {
    const params: DefaultParams = defaultParamsProvider.getParams();
    expect(params.DEPARTURE_TIME_MARGIN).toBe(900);
    expect(params.PASSENGER).toBeTruthy();
    expect(params.DRIVER).toBeFalsy();
    expect(params.TIMEZONE).toBe('Europe/Paris');
    expect(params.ALGORITHM_TYPE).toBe('PASSENGER_ORIENTED');
    expect(params.REMOTENESS).toBe(15000);
    expect(params.USE_PROPORTION).toBeTruthy();
    expect(params.PROPORTION).toBe(0.3);
    expect(params.USE_AZIMUTH).toBeTruthy();
    expect(params.AZIMUTH_MARGIN).toBe(10);
    expect(params.MAX_DETOUR_DISTANCE_RATIO).toBe(0.3);
    expect(params.MAX_DETOUR_DURATION_RATIO).toBe(0.3);
  });
});
